@@ -34,7 +34,7 @@ class EvalHook(Hook):
     rule_map = {'greater': lambda x, y: x > y, 'less': lambda x, y: x < y}
     init_value_map = {'greater': -inf, 'less': inf}
     greater_keys = ['acc', 'ap', 'ar', 'pck', 'auc']
-    less_keys = ['loss', 'epe']
+    less_keys = ['loss', 'epe', 'nme']
 
     def __init__(self,
                  dataloader,
