@@ -1,9 +1,10 @@
 from .freihand_dataset import FreiHandDataset
 from .interhand2d_dataset import InterHand2DDataset
+from .interhand3d_dataset import InterHand3DDataset
 from .onehand10k_dataset import OneHand10KDataset
 from .panoptic_dataset import PanopticDataset
 
 __all__ = [
-    'FreiHandDataset', 'InterHand2DDataset', 'OneHand10KDataset',
-    'PanopticDataset'
+    'FreiHandDataset', 'InterHand2DDataset', 'InterHand3DDataset',
+    'OneHand10KDataset', 'PanopticDataset'
 ]
