@@ -85,6 +85,7 @@ class TopDownJhmdbDataset(TopDownCocoDataset):
             ],
             dtype=np.float32).reshape((self.ann_info['num_joints'], 1))
 
+        # Adapted from COCO dataset
         self.sigmas = np.array([
             .25, 1.07, .25, .79, .79, 1.07, 1.07, .72, .72, .87, .87, .62, .62,
             .89, .89
@@ -211,7 +212,7 @@ class TopDownJhmdbDataset(TopDownCocoDataset):
             auc_nor (float): AUC normalization factor, default as 30 pixel.
 
         Returns:
-            dict: Evaluation results for evaluation metric.
+            List: Evaluation results for evaluation metric.
         """
         info_str = []
 
@@ -304,9 +305,9 @@ class TopDownJhmdbDataset(TopDownCocoDataset):
 
         Args:
             outputs (list(preds, boxes, image_path, output_heatmap))
-                :preds (np.ndarray[1,K,3]): The first two dimensions are
+                :preds (np.ndarray[N,K,3]): The first two dimensions are
                     coordinates, score is the third dimension of the array.
-                :boxes (np.ndarray[1,6]): [center[0], center[1], scale[0]
+                :boxes (np.ndarray[N,6]): [center[0], center[1], scale[0]
                     , scale[1],area, score]
                 :image_path (list[str])
                 :output_heatmap (np.ndarray[N, K, H, W]): model outpus.
@@ -330,7 +331,12 @@ class TopDownJhmdbDataset(TopDownCocoDataset):
 
         kpts = []
 
-        for preds, boxes, image_paths, _, bbox_ids in outputs:
+        for output in outputs:
+            preds = output['preds']
+            boxes = output['boxes']
+            image_paths = output['image_paths']
+            bbox_ids = output['bbox_ids']
+
             # convert 0-based index to 1-based index,
             # and get the first two dimensions.
             preds[..., :2] += 1.0
