@@ -1,3 +1,5 @@
+import math
+
 import cv2
 import numpy as np
 
@@ -6,12 +8,66 @@ from mmpose.datasets.registry import PIPELINES
 from .shared_transform import Compose
 
 
+def get_warp_matrix(theta, size_input, size_dst, size_target):
+    """Calculate the transformation matrix under the constraint of unbiased.
+    Paper ref: Huang et al. The Devil is in the Details: Delving into Unbiased
+    Data Processing for Human Pose Estimation (CVPR 2020).
+
+    Args:
+        theta (float): Rotation angle in degrees.
+        size_input (np.ndarray): Size of input image [w, h].
+        size_dst (np.ndarray): Size of output image [w, h].
+        size_target (np.ndarray): Size of ROI in input plane [w, h].
+
+    Returns:
+        matrix (np.ndarray): A matrix for transformation.
+    """
+    theta = np.deg2rad(theta)
+    matrix = np.zeros((2, 3), dtype=np.float32)
+    scale_x = size_dst[0] / size_target[0]
+    scale_y = size_dst[1] / size_target[1]
+    matrix[0, 0] = math.cos(theta) * scale_x
+    matrix[0, 1] = -math.sin(theta) * scale_x
+    matrix[0, 2] = scale_x * (-0.5 * size_input[0] * math.cos(theta) +
+                              0.5 * size_input[1] * math.sin(theta) +
+                              0.5 * size_target[0])
+    matrix[1, 0] = math.sin(theta) * scale_y
+    matrix[1, 1] = math.cos(theta) * scale_y
+    matrix[1, 2] = scale_y * (-0.5 * size_input[0] * math.sin(theta) -
+                              0.5 * size_input[1] * math.cos(theta) +
+                              0.5 * size_target[1])
+    return matrix
+
+
+def warp_affine_joints(joints, mat):
+    """Apply affine transformation defined by the transform matrix on the
+    joints.
+
+    Args:
+        joints (np.ndarray[..., 2]): Origin coordinate of joints.
+        mat (np.ndarray[3, 2]): The affine matrix.
+
+    Returns:
+        matrix (np.ndarray[..., 2]): Result coordinate of joints.
+    """
+    joints = np.array(joints)
+    shape = joints.shape
+    joints = joints.reshape(-1, 2)
+    return np.dot(
+        np.concatenate((joints, joints[:, 0:1] * 0 + 1), axis=1),
+        mat.T).reshape(shape)
+
+
 def _ceil_to_multiples_of(x, base=64):
     """Transform x to the integral multiple of the base."""
     return int(np.ceil(x / base)) * base
 
 
-def _get_multi_scale_size(image, input_size, current_scale, min_scale):
+def _get_multi_scale_size(image,
+                          input_size,
+                          current_scale,
+                          min_scale,
+                          use_udp=False):
     """Get the size for multi-scale training.
 
     Args:
@@ -19,6 +75,9 @@ def _get_multi_scale_size(image, input_size, current_scale, min_scale):
         input_size (int): Size of the image input.
         current_scale (float): Scale factor.
         min_scale (float): Minimal scale.
+        use_udp (bool): To use unbiased data processing.
+            Paper ref: Huang et al. The Devil is in the Details: Delving into
+            Unbiased Data Processing for Human Pose Estimation (CVPR 2020).
 
     Returns:
         tuple: A tuple containing multi-scale sizes.
@@ -29,8 +88,6 @@ def _get_multi_scale_size(image, input_size, current_scale, min_scale):
     """
     h, w, _ = image.shape
 
-    center = np.array([round(w / 2.0), round(h / 2.0)])
-
     # calculate the size for min_scale
     min_input_size = _ceil_to_multiples_of(min_scale * input_size, 64)
     if w < h:
@@ -38,16 +95,27 @@ def _get_multi_scale_size(image, input_size, current_scale, min_scale):
         h_resized = int(
             _ceil_to_multiples_of(min_input_size / w * h, 64) * current_scale /
             min_scale)
-        scale_w = w / 200.0
-        scale_h = h_resized / w_resized * w / 200.0
+        if use_udp:
+            scale_w = w - 1.0
+            scale_h = (h_resized - 1.0) / (w_resized - 1.0) * (w - 1.0)
+        else:
+            scale_w = w / 200.0
+            scale_h = h_resized / w_resized * w / 200.0
     else:
         h_resized = int(min_input_size * current_scale / min_scale)
         w_resized = int(
             _ceil_to_multiples_of(min_input_size / h * w, 64) * current_scale /
             min_scale)
-        scale_h = h / 200.0
-        scale_w = w_resized / h_resized * h / 200.0
-
+        if use_udp:
+            scale_h = h - 1.0
+            scale_w = (w_resized - 1.0) / (h_resized - 1.0) * (h - 1.0)
+        else:
+            scale_h = h / 200.0
+            scale_w = w_resized / h_resized * h / 200.0
+    if use_udp:
+        center = (scale_w / 2.0, scale_h / 2.0)
+    else:
+        center = np.array([round(w / 2.0), round(h / 2.0)])
     return (w_resized, h_resized), center, np.array([scale_w, scale_h])
 
 
@@ -76,6 +144,39 @@ def _resize_align_multi_scale(image, input_size, current_scale, min_scale):
     return image_resized, center, scale
 
 
+def _resize_align_multi_scale_udp(image, input_size, current_scale, min_scale):
+    """Resize the images for multi-scale training.
+
+    Args:
+        image: Input image
+        input_size (int): Size of the image input
+        current_scale (float): Current scale
+        min_scale (float): Minimal scale
+
+    Returns:
+        tuple: A tuple containing image info.
+
+        - image_resized (tuple): size of resize image
+        - center (np.ndarray): center of image
+        - scale (np.ndarray): scale
+    """
+    size_resized, _, _ = _get_multi_scale_size(image, input_size,
+                                               current_scale, min_scale, True)
+
+    _, center, scale = _get_multi_scale_size(image, input_size, min_scale,
+                                             min_scale, True)
+
+    trans = get_warp_matrix(
+        theta=0,
+        size_input=np.array(scale, dtype=np.float),
+        size_dst=np.array(size_resized, dtype=np.float) - 1.0,
+        size_target=np.array(scale, dtype=np.float))
+    image_resized = cv2.warpAffine(
+        image.copy(), trans, size_resized, flags=cv2.INTER_LINEAR)
+
+    return image_resized, center, scale
+
+
 class HeatmapGenerator:
     """Generate heatmaps for bottom-up models.
 
@@ -83,19 +184,27 @@ class HeatmapGenerator:
         num_joints (int): Number of keypoints
         output_res (int): Size of feature map
         sigma (int): Sigma of the heatmaps.
+        use_udp (bool): To use unbiased data processing.
+            Paper ref: Huang et al. The Devil is in the Details: Delving into
+            Unbiased Data Processing for Human Pose Estimation (CVPR 2020).
     """
 
-    def __init__(self, output_res, num_joints, sigma=-1):
+    def __init__(self, output_res, num_joints, sigma=-1, use_udp=False):
         self.output_res = output_res
         self.num_joints = num_joints
         if sigma < 0:
             sigma = self.output_res / 64
         self.sigma = sigma
         size = 6 * sigma + 3
-        x = np.arange(0, size, 1, np.float32)
-        y = x[:, None]
-        x0, y0 = 3 * sigma + 1, 3 * sigma + 1
-        self.g = np.exp(-((x - x0)**2 + (y - y0)**2) / (2 * sigma**2))
+        self.use_udp = use_udp
+        if use_udp:
+            self.x = np.arange(0, size, 1, np.float32)
+            self.y = self.x[:, None]
+        else:
+            x = np.arange(0, size, 1, np.float32)
+            y = x[:, None]
+            x0, y0 = 3 * sigma + 1, 3 * sigma + 1
+            self.g = np.exp(-((x - x0)**2 + (y - y0)**2) / (2 * sigma**2))
 
     def __call__(self, joints):
         """Generate heatmaps."""
@@ -110,10 +219,18 @@ class HeatmapGenerator:
                        x >= self.output_res or y >= self.output_res:
                         continue
 
-                    ul = int(np.round(x - 3 * sigma - 1)), int(
-                        np.round(y - 3 * sigma - 1))
-                    br = int(np.round(x + 3 * sigma + 2)), int(
-                        np.round(y + 3 * sigma + 2))
+                    if self.use_udp:
+                        x0 = 3 * sigma + 1 + pt[0] - x
+                        y0 = 3 * sigma + 1 + pt[1] - y
+                        g = np.exp(-((self.x - x0)**2 + (self.y - y0)**2) /
+                                   (2 * sigma**2))
+                    else:
+                        g = self.g
+
+                    ul = int(np.round(x - 3 * sigma -
+                                      1)), int(np.round(y - 3 * sigma - 1))
+                    br = int(np.round(x + 3 * sigma +
+                                      2)), int(np.round(y + 3 * sigma + 2))
 
                     c, d = max(0, -ul[0]), min(br[0], self.output_res) - ul[0]
                     a, b = max(0, -ul[1]), min(br[1], self.output_res) - ul[1]
@@ -121,8 +238,8 @@ class HeatmapGenerator:
                     cc, dd = max(0, ul[0]), min(br[0], self.output_res)
                     aa, bb = max(0, ul[1]), min(br[1], self.output_res)
                     hms[idx, aa:bb,
-                        cc:dd] = np.maximum(hms[idx, aa:bb, cc:dd],
-                                            self.g[a:b, c:d])
+                        cc:dd] = np.maximum(hms[idx, aa:bb, cc:dd], g[a:b,
+                                                                      c:d])
         return hms
 
 
@@ -220,14 +337,23 @@ class BottomUpRandomAffine:
         scale_type: wrt ``long`` or ``short`` length of the image.
         trans_factor: Translation factor.
         scale_aware_sigma: Option to use scale-aware sigma
+        use_udp (bool): To use unbiased data processing.
+            Paper ref: Huang et al. The Devil is in the Details: Delving into
+            Unbiased Data Processing for Human Pose Estimation (CVPR 2020).
     """
 
-    def __init__(self, rot_factor, scale_factor, scale_type, trans_factor):
+    def __init__(self,
+                 rot_factor,
+                 scale_factor,
+                 scale_type,
+                 trans_factor,
+                 use_udp=False):
         self.max_rotation = rot_factor
         self.min_scale = scale_factor[0]
         self.max_scale = scale_factor[1]
         self.scale_type = scale_type
         self.trans_factor = trans_factor
+        self.use_udp = use_udp
 
     @staticmethod
     def _get_affine_matrix(center, scale, res, rot=0):
@@ -256,16 +382,6 @@ class BottomUpRandomAffine:
             t = np.dot(t_inv, np.dot(rot_mat, np.dot(t_mat, t)))
         return t
 
-    @staticmethod
-    def _affine_joints(joints, mat):
-        """Affine the joints by the transform matrix."""
-        joints = np.array(joints)
-        shape = joints.shape
-        joints = joints.reshape(-1, 2)
-        return np.dot(
-            np.concatenate((joints, joints[:, 0:1] * 0 + 1), axis=1),
-            mat.T).reshape(shape)
-
     def __call__(self, results):
         """Perform data augmentation with random scaling & rotating."""
         image, mask, joints = results['img'], results['mask'], results[
@@ -282,8 +398,10 @@ class BottomUpRandomAffine:
                                                     self.output_size)
 
         height, width = image.shape[:2]
-
-        center = np.array((width / 2, height / 2))
+        if self.use_udp:
+            center = np.array(((width - 1.0) / 2, (height - 1.0) / 2))
+        else:
+            center = np.array((width / 2, height / 2))
         if self.scale_type == 'long':
             scale = max(height, width) / 1.0
         elif self.scale_type == 'short':
@@ -303,25 +421,53 @@ class BottomUpRandomAffine:
 
             center[0] += dx
             center[1] += dy
+        if self.use_udp:
+            for i, _output_size in enumerate(self.output_size):
+                trans = get_warp_matrix(
+                    theta=aug_rot,
+                    size_input=center * 2.0,
+                    size_dst=np.array(
+                        (_output_size, _output_size), dtype=np.float) - 1.0,
+                    size_target=np.array((scale, scale), dtype=np.float))
+                mask[i] = cv2.warpAffine(
+                    (mask[i] * 255).astype(np.uint8),
+                    trans, (int(_output_size), int(_output_size)),
+                    flags=cv2.INTER_LINEAR) / 255
+                mask[i] = (mask[i] > 0.5).astype(np.float32)
+                joints[i][:, :, 0:2] = \
+                    warp_affine_joints(joints[i][:, :, 0:2].copy(), trans)
+                if results['ann_info']['scale_aware_sigma']:
+                    joints[i][:, :, 3] = joints[i][:, :, 3] / aug_scale
+            mat_input = get_warp_matrix(
+                theta=aug_rot,
+                size_input=center * 2.0,
+                size_dst=np.array(
+                    (self.input_size, self.input_size), dtype=np.float) - 1.0,
+                size_target=np.array((scale, scale), dtype=np.float))
+            image = cv2.warpAffine(
+                image,
+                mat_input, (int(self.input_size), int(self.input_size)),
+                flags=cv2.INTER_LINEAR)
+        else:
+            for i, _output_size in enumerate(self.output_size):
+                mat_output = self._get_affine_matrix(center, scale,
+                                                     (_output_size,
+                                                      _output_size),
+                                                     aug_rot)[:2]
+                mask[i] = cv2.warpAffine(
+                    (mask[i] * 255).astype(np.uint8), mat_output,
+                    (_output_size, _output_size)) / 255
+                mask[i] = (mask[i] > 0.5).astype(np.float32)
 
-        for i, _output_size in enumerate(self.output_size):
-            mat_output = self._get_affine_matrix(center, scale,
-                                                 (_output_size, _output_size),
-                                                 aug_rot)[:2]
-            mask[i] = cv2.warpAffine(
-                (mask[i] * 255).astype(np.uint8), mat_output,
-                (_output_size, _output_size)) / 255
-            mask[i] = (mask[i] > 0.5).astype(np.float32)
-
-            joints[i][:, :, 0:2] = self._affine_joints(joints[i][:, :, 0:2],
-                                                       mat_output)
-            if results['ann_info']['scale_aware_sigma']:
-                joints[i][:, :, 3] = joints[i][:, :, 3] / aug_scale
-        mat_input = self._get_affine_matrix(center, scale,
-                                            (self.input_size, self.input_size),
-                                            aug_rot)[:2]
-        image = cv2.warpAffine(image, mat_input,
-                               (self.input_size, self.input_size))
+                joints[i][:, :, 0:2] = \
+                    warp_affine_joints(joints[i][:, :, 0:2], mat_output)
+                if results['ann_info']['scale_aware_sigma']:
+                    joints[i][:, :, 3] = joints[i][:, :, 3] / aug_scale
+            mat_input = self._get_affine_matrix(center, scale,
+                                                (self.input_size,
+                                                 self.input_size), aug_rot)[:2]
+            image = cv2.warpAffine(image, mat_input,
+                                   (self.input_size, self.input_size))
 
         results['img'], results['mask'], results[
             'joints'] = image, mask, joints
@@ -336,16 +482,20 @@ class BottomUpGenerateTarget:
     Args:
         sigma (int): Sigma of heatmap Gaussian
         max_num_people (int): Maximum number of people in an image
+        use_udp (bool): To use unbiased data processing.
+            Paper ref: Huang et al. The Devil is in the Details: Delving into
+            Unbiased Data Processing for Human Pose Estimation (CVPR 2020).
     """
 
-    def __init__(self, sigma, max_num_people):
+    def __init__(self, sigma, max_num_people, use_udp=False):
         self.sigma = sigma
         self.max_num_people = max_num_people
+        self.use_udp = use_udp
 
     def _generate(self, num_joints, heatmap_size):
         """Get heatmap generator and joint encoder."""
         heatmap_generator = [
-            HeatmapGenerator(output_size, num_joints, self.sigma)
+            HeatmapGenerator(output_size, num_joints, self.sigma, self.use_udp)
             for output_size in heatmap_size
         ]
         joints_encoder = [
@@ -387,12 +537,16 @@ class BottomUpGetImgSize:
     Args:
         test_scale_factor (List[float]): Multi scale
         current_scale (int): default 1
+        use_udp (bool): To use unbiased data processing.
+            Paper ref: Huang et al. The Devil is in the Details: Delving into
+            Unbiased Data Processing for Human Pose Estimation (CVPR 2020).
     """
 
-    def __init__(self, test_scale_factor, current_scale=1):
+    def __init__(self, test_scale_factor, current_scale=1, use_udp=False):
         self.test_scale_factor = test_scale_factor
         self.min_scale = min(test_scale_factor)
         self.current_scale = current_scale
+        self.use_udp = use_udp
 
     def __call__(self, results):
         """Get multi-scale image sizes for bottom-up."""
@@ -400,7 +554,6 @@ class BottomUpGetImgSize:
         img = results['img']
 
         h, w, _ = img.shape
-        center = np.array([round(w / 2.0), round(h / 2.0)])
 
         # calculate the size for min_scale
         min_input_size = _ceil_to_multiples_of(self.min_scale * input_size, 64)
@@ -410,17 +563,28 @@ class BottomUpGetImgSize:
             h_resized = int(
                 _ceil_to_multiples_of(min_input_size / w * h, 64) *
                 self.current_scale / self.min_scale)
-            scale_w = w / 200.0
-            scale_h = h_resized / w_resized * w / 200.0
+            if self.use_udp:
+                scale_w = w - 1.0
+                scale_h = (h_resized - 1.0) / (w_resized - 1.0) * (w - 1.0)
+            else:
+                scale_w = w / 200.0
+                scale_h = h_resized / w_resized * w / 200.0
         else:
             h_resized = int(min_input_size * self.current_scale /
                             self.min_scale)
             w_resized = int(
                 _ceil_to_multiples_of(min_input_size / h * w, 64) *
                 self.current_scale / self.min_scale)
-            scale_h = h / 200.0
-            scale_w = w_resized / h_resized * h / 200.0
-
+            if self.use_udp:
+                scale_h = h - 1.0
+                scale_w = (w_resized - 1.0) / (h_resized - 1.0) * (h - 1.0)
+            else:
+                scale_h = h / 200.0
+                scale_w = w_resized / h_resized * h / 200.0
+        if self.use_udp:
+            center = (scale_w / 2.0, scale_h / 2.0)
+        else:
+            center = np.array([round(w / 2.0), round(h / 2.0)])
         results['ann_info']['test_scale_factor'] = self.test_scale_factor
         results['ann_info']['base_size'] = (w_resized, h_resized)
         results['ann_info']['center'] = center
@@ -435,10 +599,17 @@ class BottomUpResizeAlign:
 
     Args:
         transforms (List): ToTensor & Normalize
+        use_udp (bool): To use unbiased data processing.
+            Paper ref: Huang et al. The Devil is in the Details: Delving into
+            Unbiased Data Processing for Human Pose Estimation (CVPR 2020).
     """
 
-    def __init__(self, transforms):
+    def __init__(self, transforms, use_udp=False):
         self.transforms = Compose(transforms)
+        if use_udp:
+            self._resize_align_multi_scale = _resize_align_multi_scale_udp
+        else:
+            self._resize_align_multi_scale = _resize_align_multi_scale
 
     def __call__(self, results):
         """Resize multi-scale size and align transform for bottom-up."""
@@ -448,7 +619,7 @@ class BottomUpResizeAlign:
 
         for _, s in enumerate(sorted(test_scale_factor, reverse=True)):
             _results = results.copy()
-            image_resized, _, _ = _resize_align_multi_scale(
+            image_resized, _, _ = self._resize_align_multi_scale(
                 _results['img'], input_size, s, min(test_scale_factor))
             _results['img'] = image_resized
             _results = self.transforms(_results)
