@@ -437,6 +437,11 @@ class TopDownGenerateTarget:
 
             tmp_size = factor * 3
 
+            # prepare for gaussian
+            size = 2 * tmp_size + 1
+            x = np.arange(0, size, 1, np.float32)
+            y = x[:, None]
+
             for joint_id in range(num_joints):
                 feat_stride = (image_size - 1.0) / (heatmap_size - 1.0)
                 mu_x = int(joints_3d[joint_id][0] / feat_stride[0] + 0.5)
@@ -451,9 +456,6 @@ class TopDownGenerateTarget:
                     continue
 
                 # # Generate gaussian
-                size = 2 * tmp_size + 1
-                x = np.arange(0, size, 1, np.float32)
-                y = x[:, None]
                 mu_x_ac = joints_3d[joint_id][0] / feat_stride[0]
                 mu_y_ac = joints_3d[joint_id][1] / feat_stride[1]
                 x0 = y0 = size // 2
@@ -562,7 +564,7 @@ class TopDownGenerateTarget:
                 factors = self.sigma
                 channel_factor = 1
             if isinstance(factors, list):
-                num_factors = len(self.kernel)
+                num_factors = len(factors)
                 cfg = results['ann_info']
                 num_joints = cfg['num_joints']
                 W, H = cfg['heatmap_size']
@@ -589,3 +591,57 @@ class TopDownGenerateTarget:
         results['target_weight'] = target_weight
 
         return results
+
+
+@PIPELINES.register_module()
+class TopDownGenerateTargetRegression():
+    """Generate the target regression vector (coordinates).
+
+    Required keys: 'joints_3d', 'joints_3d_visible', 'ann_info'. Modified keys:
+    'target', and 'target_weight'.
+    """
+
+    def __init__(self):
+        pass
+
+    def _generate_target(self, cfg, joints_3d, joints_3d_visible):
+        """Generate the target regression vector.
+
+        Args:
+            cfg (dict): data config
+            joints_3d: np.ndarray([num_joints, 3])
+            joints_3d_visible: np.ndarray([num_joints, 3])
+        Returns:
+             target, target_weight(1: visible, 0: invisible)
+        """
+        image_size = cfg['image_size']
+        joint_weights = cfg['joint_weights']
+        use_different_joint_weights = cfg['use_different_joint_weights']
+
+        mask = (joints_3d[:, 0] >= 0) * (
+            joints_3d[:, 0] <= image_size[0] - 1) * (joints_3d[:, 1] >= 0) * (
+                joints_3d[:, 1] <= image_size[1] - 1)
+
+        target = joints_3d[:, :2] / image_size
+
+        target = target.astype(np.float32)
+        target_weight = joints_3d_visible[:, :2] * mask[:, None]
+
+        if use_different_joint_weights:
+            target_weight = np.multiply(target_weight, joint_weights)
+
+        return target, target_weight
+
+    def __call__(self, results):
+        """Generate the target heatmap."""
+        joints_3d = results['joints_3d']
+        joints_3d_visible = results['joints_3d_visible']
+
+        target, target_weight = self._generate_target(results['ann_info'],
+                                                      joints_3d,
+                                                      joints_3d_visible)
+
+        results['target'] = target
+        results['target_weight'] = target_weight
+
+        return results
