@@ -1,12 +1,14 @@
 from .backbones import *  # noqa
-from .builder import build_backbone, build_head, build_loss, build_posenet
+from .builder import (build_backbone, build_head, build_loss, build_neck,
+                      build_posenet)
 from .detectors import *  # noqa
 from .keypoint_heads import *  # noqa
 from .losses import *  # noqa
 from .mesh_heads import *  # noqa
+from .necks import *  # noqa
 from .registry import BACKBONES, HEADS, LOSSES, POSENETS
 
 __all__ = [
     'BACKBONES', 'HEADS', 'LOSSES', 'POSENETS', 'build_backbone', 'build_head',
-    'build_loss', 'build_posenet'
+    'build_loss', 'build_posenet', 'build_neck'
 ]
