@@ -509,9 +509,10 @@ class ResNet_top(nn.Module):
 
 @BACKBONES.register_module()
 class RSN(BaseBackbone):
-    """Residual Steps Network backbone.
-              Paper ref: Cai et al. "Learning Delicate Local Representations
-              for Multi-Person Pose Estimation" (ECCV 2020).
+    """Residual Steps Network backbone. Paper ref: Cai et al. "Learning
+    Delicate Local Representations for Multi-Person Pose Estimation" (ECCV
+    2020).
+
     Args:
         unit_channels (int): Number of Channels in an upsample unit.
             Default: 256
