@@ -1,12 +1,17 @@
 import numpy as np
 import torch
-from smplx import SMPL
 
 from .. import builder
 from ..mesh_heads.discriminator import SMPLDiscriminator
 from ..registry import POSENETS
 from .base import BasePose
 
+try:
+    from smplx import SMPL
+    has_smpl = True
+except (ImportError, ModuleNotFoundError):
+    has_smpl = False
+
 
 def set_requires_grad(nets, requires_grad=False):
     """Set requies_grad for all the networks.
@@ -53,6 +58,8 @@ class ParametricMesh(BasePose):
                  pretrained=None):
         super().__init__()
 
+        assert has_smpl, 'Please install smplx to use SMPL.'
+
         self.backbone = builder.build_backbone(backbone)
         self.mesh_head = builder.build_head(mesh_head)
         self.generator = torch.nn.Sequential(self.backbone, self.mesh_head)
