@@ -1,8 +1,9 @@
 from .mesh_loss import GANLoss, MeshLoss
 from .mse_loss import JointsMSELoss, JointsOHKMMSELoss
 from .multi_loss_factory import AELoss, HeatmapLoss, MultiLossFactory
+from .regression_loss import SmoothL1Loss
 
 __all__ = [
     'JointsMSELoss', 'JointsOHKMMSELoss', 'HeatmapLoss', 'AELoss',
-    'MultiLossFactory', 'MeshLoss', 'GANLoss'
+    'MultiLossFactory', 'MeshLoss', 'GANLoss', 'SmoothL1Loss'
 ]
